//--------------------------------------------------------------------------
// Deployable Generator
//--------------------------------------------------------------------------

datablock SensorData(SensorCommPulseObj)
{
   detects = true;
   detectsUsingLOS = true;
   detectsPassiveJammed = false;
   detectsActiveJammed = false;
   detectsCloaked = false;
   detectionPings = true;
   detectRadius = 750;
};

datablock StaticShapeData(DeployedCommCenter) : StaticShapeDamageProfile {
   className = Sensor;
   catagory = "Sensors";
   shapeFile = "sensor_pulse_large.dts";
   maxDamage = 3.5;
   destroyedLevel = 3.5;
   disabledLevel = 2.85;
   explosion      = ShapeExplosion;
   expDmgRadius = 10.0;
   expDamage = 0.5;
   expImpulse = 2000.0;

   dynamicType = $TypeMasks::StaticShapeObjectType;
   isShielded = false;
   energyPerDamagePoint = 33;
   maxEnergy = 110;
   rechargeRate = 0.31;
   ambientThreadPowered = true;
   humSound = SensorHumSound;

   cmdCategory = "Support";
   cmdIcon = CMDSensorIcon;
   cmdMiniIconName = "commander/MiniIcons/com_sensor_grey";
   targetNameTag = 'Comm. Center';
   targetTypeTag = '';
   sensorData = SensorCommPulseObj;
   sensorRadius = SensorCommPulseObj.detectRadius;
   sensorColor = "255 194 9";

   debrisShapeName = "debris_generic.dts";
   debris = StaticShapeDebris;
};


datablock ShapeBaseImageData(CommCenterImage) {
	mass = 20;
	emap = true;
	shapeFile = "stackable1s.dts";
	item = CommCenterDeployable;
	mountPoint = 1;
	offset = "0 0 0";
	deployed = DeployedCommCenter;
	heatSignature = 0;

	stateName[0] = "Idle";
	stateTransitionOnTriggerDown[0] = "Activate";

	stateName[1] = "Activate";
	stateScript[1] = "onActivate";
	stateTransitionOnTriggerUp[1] = "Idle";

	isLarge = true;
	maxDepSlope = 360;
	deploySound = ItemPickupSound;

	minDeployDis = 2;
	maxDeployDis = 5;
};

datablock ItemData(CommCenterDeployable) {
	className = Pack;
	catagory = "Deployables";
	shapeFile = "stackable1s.dts";
	mass = 5.0;

	hasLight = true;
	lightType = "PulsingLight";
	lightColor = "0.1 0.8 0.8 1.0";
	lightTime = "1000";
	lightRadius = "3";

	elasticity = 0.2;
	friction = 0.6;
	pickupRadius = 3;
	rotate = true;
	image = "CommCenterImage";
	pickUpName = "a Power Plant";
	heatSignature = 0;
	emap = true;
};

function CommCenterImage::testNoTerrainFound(%item) {
	// don't check this for non-Landspike turret deployables
}

function CommCenterDeployable::onPickup(%this, %obj, %shape, %amount) {
	// created to prevent console errors
}

function CommCenterImage::onDeploy(%item, %plyr, %slot) {

       if($BuildingCount::PowerPlant[%plyr.client.team] <= 0) {
       messageclient(%plyr.client, 'MsgClient', '\c5Tech Error, Power Plant Required.');
       return;
       }
       if($BuildingCount::TiberiumRefinery[%plyr.client.team] <= 0) {
       messageclient(%plyr.client, 'MsgClient', '\c5Tech Error, Tiberium Refinery Required.');
       return;
       }

	%className = "StaticShape";

	%playerVector = vectorNormalize(getWord(%plyr.getEyeVector(),1) SPC -1 * getWord(%plyr.getEyeVector(),0) SPC "0");

	if (vAbs(floorVec(%item.surfaceNrm,100)) $= "0 0 1")
		%item.surfaceNrm2 = vectorScale(%playerVector,-1);
	else
		%item.surfaceNrm2 = vectorNormalize(vectorCross(%item.surfaceNrm,"0 0 1"));

	%rot = fullRot(%item.surfaceNrm,%item.surfaceNrm2);

	%item.surfacePt = vectorAdd(%item.surfacePt,vectorScale(vectorNormalize(vectorCross(%item.surfaceNrm,%item.surfaceNrm2)),2));

	%deplObj = new (%className)() {
		dataBlock = DeployedCommCenter;
        scale = "2.5 2.5 2.5";
		deployed = true;
	};

	// set orientation
	%deplObj.setTransform(%item.surfacePt SPC %rot);

	// set team, owner, and handle
	%deplObj.team = %plyr.client.Team;

	// set power frequency
	%deplObj.powerFreq = %plyr.powerFreq;
	setTargetName(%deplObj.target,addTaggedString("Communications Center"));

	// set power
	%deplObj.setSelfPowered();

	// set the sensor group if it needs one
	if (%deplObj.getTarget() != -1)
		setTargetSensorGroup(%deplObj.getTarget(), %plyr.client.team);

	// place the deployable in the MissionCleanup/Deployables group (AI reasons)
	addToDeployGroup(%deplObj);

	//let the AI know as well...
	AIDeployObject(%plyr.client, %deplObj);

	// play the deploy sound
	serverPlay3D(%item.deploySound, %deplObj.getTransform());

	// increment the team count for this deployed object
	$TeamDeployedCount[%plyr.team, %item.item]++;

	addDSurface(%item.surface,%deplObj);

	%deplObj.playThread($PowerThread,"Power");

    $BuildingCount::CommCenter[%deplObj.team]++;
    $Power[%deplObj.team] = $Power[%deplObj.team] - 150; //Add 500 Power From The Team Count
    CheckPowerLowLevel(%deplObj.team);
    %deplObj.team.CommCenterCount++;

	// take the deployable off the player's back and out of inventory
	%plyr.unmountImage(%slot);
	%plyr.decInventory(%item.item, 1);

	// add to power list
	$PowerList = listAdd($PowerList,%deplObj,-1);

      %count = ClientGroup.getCount();
      for(%i = 0; %i < %count; %i++) {
         %cl = ClientGroup.getObject(%i);
         if(%cl.team == %deplObj.team) {
         messageclient(%cl, 'MsgClient', "\c5Base Update: Construction Complete, Communications Center.");
            if(%deplObj.team == 1 || %deplObj.team == 3)
            messageclient(%cl, 'MsgClient', "\c5Superweapon Update: Airstrike Charging.");
         }
      }
    if($BuildingCount::CommCenter[%deplObj.team] == 1) { //who has the last laugh now huh?
    //this ensures that there is only 1 Airstrike availible
       if(%deplObj.team == 1 || %deplObj.team == 3) {
          $Superweapon::AirstrikeReady[%deplObj.team] = 0;
          $SuperweaponTimer::Airstrike[%deplObj.team] = 120;
          schedule(1000,0,"DoAirTimer", %deplObj.team);
       }
    }
    echo("C&C: Team "@%deplObj.team@" Built a Comm. Center at "@%deplObj.getPosition()@".");
	return %deplObj;
}

function DeployedCommCenter::onDestroyed(%this,%obj,%prevState) {
      $BuildingCount::CommCenter[%obj.team]--;
      %count = ClientGroup.getCount();
      for(%i = 0; %i < %count; %i++) {
         %cl = ClientGroup.getObject(%i);
         if(%cl.team == %obj.team) {
         messageclient(%cl, 'MsgClient', "\c5Base Update: Building Lost, Communications Center.");
            if($BuildingCount::CommCenter[%obj.team] == 0) {
               messageclient(%cl, 'MsgClient', "\c5Superweapon Update: Airstrike Disabled.");
               $Superweapon::AirstrikeReady[%obj.team] = 0;
            }
         }
      }
      if($BuildingCount::CommCenter[%obj.team] == 0) { //disable the airstrike
         $SuperweaponTimer::Airstrike[%obj.team] = 120;
      }
   $TeamDeployedCount[%obj.team,CommCenterDeployable]--;
   $Power[%obj.team] = $Power[%obj.team] + 150; //Remove 250 Power From The Team Count
   CheckPowerGoodLevel(%obj.team);
   %obj.team.CommCenterCount--;
   remDSurface(%obj);
   %obj.schedule(500,"delete");
   if($Game::PlayingCnC) {
   $TeamScore[%obj.lastDamagedByTeam] = $TeamScore[%obj.lastDamagedByTeam] + $CNC::Score[CommCenter];
   }
   Parent::onDestroyed(%data,%obj,%prevState);
}

function CommCenterImage::onMount(%data,%obj,%node) {
	%obj.hasGen = true; // set for gencheck
}

function CommCenterImage::onUnmount(%data,%obj,%node) {
	%obj.hasGen = "";
}

function DoAirTimer(%team) {
   if($BuildingCount::CommCenter[%team] == 0) {
      schedule(1000,0,"DoAirTimer", %team);
      return;
   }
   if($SuperweaponTimer::Airstrike[%team] == 0) {
      if($Superweapon::AirstrikeReady[%team] == 0) {
         ASReady(%team);
      }
      schedule(1000,0,"DoAirTimer", %team);
      return;
   }
   if($Power[%team] < 0) {
      schedule(1000,0,"DoAirTimer", %team);
      return;
   }
   $SuperweaponTimer::Airstrike[%team]--;
   schedule(1000,0,"DoAirTimer", %team);
}

function ASReady(%team) {
   %count = ClientGroup.getCount();
   for(%i = 0; %i < %count; %i++) {
      %cl = ClientGroup.getObject(%i);
      if(%cl.team == %team) {
      messageclient(%cl, 'MsgClient', "\c5Superweapon Update: Airstrike Ready.");
      }
   }
   $Superweapon::AirstrikeReady[%team] = 1;
}

function PrepAirRaid(%obj, %pos) {
   $Superweapon::AirstrikeReady[%obj.team] = 0;
   $SuperweaponTimer::Airstrike[%obj.team] = 120;
   %count = ClientGroup.getCount();
   for(%i = 0; %i < %count; %i++) {
      %cl = ClientGroup.getObject(%i);
      if(%cl.team == %obj.team) {
      messageclient(%cl, 'MsgClient', "\c5Orca Bomber Squad: Bomb Doors Opening.");
      }
      else {
      messageclient(%cl, 'MsgClient', "\c5WARNING: GDI Bombing Squadron Detected. ~wfx/misc/red_alert_short.wav");
      }
   }
   schedule(7500,0,"DropBombs",%obj, %pos);
}

function DropBombs(%obj, %pos) {
   %count = ClientGroup.getCount();
   for(%i = 0; %i < %count; %i++) {
      %cl = ClientGroup.getObject(%i);
      if(%cl.team == %obj.team) {
      messageclient(%cl, 'MsgClient', "\c5Orca Bomber Squad: Bombs Away!.");
      }
      else {
      messageclient(%cl, 'MsgClient', "\c5WARNING: Bombing Run Detected!!! ~wfx/misc/red_alert.wav");
      }
   }
   for(%i = 0; %i < 6; %i++) {
      %posY = vectorAdd(%pos, ""@getRandom(1,10)@" "@getRandom(1,10)@" "@%i * 20@"");
      %bombz = new (BombProjectile)() {
      dataBlock = BomberBomb;
      initialDirection = "0 0 -100";
      initialPosition = vectorAdd(%posY, "0 0 300");
      sourceObject = %obj;
      damageFactor = 1;
      };
      MissionCleanup.Add(%bombz);
   }
}
