//--------------------------------------------------------------------------
// Deployable Generator
//--------------------------------------------------------------------------

datablock StaticShapeData(DeployedTiberiumRefinery) : StaticShapeDamageProfile {
	className = "TiberiumRefinery";
	shapeFile = "stackable3l.dts"; // dmiscf.dts, alternate
	maxDamage = 5.0;
	destroyedLevel = 5.0;
	disabledLevel = 5.0;
	mass = 1.2;
	elasticity = 0.1;
	friction = 0.9;
	collideable = 1;
	pickupRadius = 1;
	sticky=false;

	impulse = 5000;

	hasLight = true;
	lightType = "PulsingLight";
	lightColor = "0.1 0.8 0.8 1.0";
	lightTime = "100";
	lightRadius = "3";

	explosion      = HandGrenadeExplosion;
	expDmgRadius = 3.0;
	expDamage = 0.1;
	expImpulse = 200.0;
	dynamicType = $TypeMasks::StaticShapeObjectType;
	deployedObject = true;
	cmdCategory = "DSupport";
	cmdIcon = CMDSensorIcon;
	cmdMiniIconName = "commander/MiniIcons/com_deploymotionsensor";

	targetNameTag = 'Tiberium Refinery';
	targetTypeTag = '';
	deployAmbientThread = true;
	debrisShapeName = "debris_generic_small.dts";
	debris = DeployableDebris;
	heatSignature = 0;
};


datablock ShapeBaseImageData(TiberiumRefineryImage) {
	mass = 20;
	emap = true;
	shapeFile = "stackable1s.dts";
	item = TiberiumRefineryDeployable;
	mountPoint = 1;
	offset = "0 0 0";
	deployed = DeployedTiberiumRefinery;
	heatSignature = 0;

	stateName[0] = "Idle";
	stateTransitionOnTriggerDown[0] = "Activate";

	stateName[1] = "Activate";
	stateScript[1] = "onActivate";
	stateTransitionOnTriggerUp[1] = "Idle";

	isLarge = true;
	maxDepSlope = 360;
	deploySound = ItemPickupSound;

	minDeployDis = 2;
	maxDeployDis = 5;
};

datablock ItemData(TiberiumRefineryDeployable) {
	className = Pack;
	catagory = "Deployables";
	shapeFile = "stackable1s.dts";
	mass = 5.0;

	hasLight = true;
	lightType = "PulsingLight";
	lightColor = "0.1 0.8 0.8 1.0";
	lightTime = "1000";
	lightRadius = "3";

	elasticity = 0.2;
	friction = 0.6;
	pickupRadius = 3;
	rotate = true;
	image = "TiberiumRefineryImage";
	pickUpName = "a Power Plant";
	heatSignature = 0;
	emap = true;
};

function TiberiumRefineryImage::testNoTerrainFound(%item) {
	// don't check this for non-Landspike turret deployables
}

function TiberiumRefineryDeployable::onPickup(%this, %obj, %shape, %amount) {
	// created to prevent console errors
}

   
//Eh For nows
function Add700Loop(%obj) {
    if(!isObject(%obj)) {
    return;
    }
    %count = ClientGroup.getCount();
    for(%i = 0; %i < %count; %i++) {
       %cl = ClientGroup.getObject(%i);
       if(%cl.team == %obj.team) {
          %cl.credits = %cl.credits + 700;
          messageclient(%cl, 'MsgClient', "\c5Tiberium Processed, you now have "@%cl.credits@" credits.");
       }
    }
    schedule(60000,0,"Add700Loop",%obj);
}
   
function TiberiumRefineryImage::onDeploy(%item, %plyr, %slot) {
    if($BuildingCount::PowerPlant[%plyr.client.team] <= 0) {
    messageclient(%plyr.client, 'MsgClient', '\c5Tech Error, Power Plant Required.');
    return;
    }

	%className = "StaticShape";

	%playerVector = vectorNormalize(getWord(%plyr.getEyeVector(),1) SPC -1 * getWord(%plyr.getEyeVector(),0) SPC "0");

	if (vAbs(floorVec(%item.surfaceNrm,100)) $= "0 0 1")
		%item.surfaceNrm2 = vectorScale(%playerVector,-1);
	else
		%item.surfaceNrm2 = vectorNormalize(vectorCross(%item.surfaceNrm,"0 0 1"));

	%rot = fullRot(%item.surfaceNrm,%item.surfaceNrm2);

	%item.surfacePt = vectorAdd(%item.surfacePt,vectorScale(vectorNormalize(vectorCross(%item.surfaceNrm,%item.surfaceNrm2)),2));

	%deplObj = new (%className)() {
		dataBlock = DeployedTiberiumRefinery;
		scale = "4.5 4.5 4.5";
		deployed = true;
	};

	// set orientation
	%deplObj.setTransform(%item.surfacePt SPC %rot);

	// set team, owner, and handle
	%deplObj.team = %plyr.client.Team;

	// set power frequency
	%deplObj.powerFreq = %plyr.powerFreq;
	setTargetName(%deplObj.target,addTaggedString("Tiberium Refinery"));

	// set power
	%deplObj.setSelfPowered();

	// set the sensor group if it needs one
	if (%deplObj.getTarget() != -1)
		setTargetSensorGroup(%deplObj.getTarget(), %plyr.client.team);

	// place the deployable in the MissionCleanup/Deployables group (AI reasons)
	addToDeployGroup(%deplObj);

	//let the AI know as well...
	AIDeployObject(%plyr.client, %deplObj);

	// play the deploy sound
	serverPlay3D(%item.deploySound, %deplObj.getTransform());

	// increment the team count for this deployed object
	$TeamDeployedCount[%plyr.team, %item.item]++;

	addDSurface(%item.surface,%deplObj);

	%deplObj.playThread($PowerThread,"Power");

    $BuildingCount::TiberiumRefinery[%deplObj.team]++;
    $Power[%deplObj.team] = $Power[%deplObj.team] - 50;

	// take the deployable off the player's back and out of inventory
	%plyr.unmountImage(%slot);
	%plyr.decInventory(%item.item, 1);

	// add to power list
	$PowerList = listAdd($PowerList,%deplObj,-1);

      %count = ClientGroup.getCount();
      for(%i = 0; %i < %count; %i++) {
         %cl = ClientGroup.getObject(%i);
         if(%cl.team == %deplObj.team) {
         messageclient(%cl, 'MsgClient', "\c5Base Update: Construction Complete, Tiberium Refinery.");
         }
      }
    CheckPowerLowLevel(%deplObj.team);

    Add700Loop(%deplObj);
    echo("C&C: Team "@%deplObj.team@" Built a Tiberium Refinery at "@%deplObj.getPosition()@".");
	return %deplObj;
}

function DeployedTiberiumRefinery::onDestroyed(%this,%obj,%prevState) {

      %count = ClientGroup.getCount();
      for(%i = 0; %i < %count; %i++) {
         %cl = ClientGroup.getObject(%i);
         if(%cl.team == %obj.team) {
         messageclient(%cl, 'MsgClient', "\c5Base Update: Building Lost, Tiberium Refinery.");
         cancel(%cl.TibLoop);
         }
      }
    $Power[%obj.team] = $Power[%obj.team] + 50;
    CheckPowerGoodLevel(%obj.team);
    $BuildingCount::TiberiumRefinery[%obj.team]--;
   $TeamDeployedCount[%obj.team,TiberiumRefineryDeployable]--;
   remDSurface(%obj);
   %obj.schedule(500,"delete");
   if($Game::PlayingCnC) {
   $TeamScore[%obj.lastDamagedByTeam] = $TeamScore[%obj.lastDamagedByTeam] + $CNC::Score[TiberiumRefinery];
   }
   Parent::onDestroyed(%data,%obj,%prevState);
}

function TiberiumRefineryImage::onMount(%data,%obj,%node) {
	%obj.hasGen = true; // set for gencheck
}

function TiberiumRefineryImage::onUnmount(%data,%obj,%node) {
	%obj.hasGen = "";
}
